using Components;
using InternetShopContracts.DataBindingModels;
using InternetShopContracts.DataSearchModels;
using InternetShopContracts.DataViewModels;
using InternetShopContracts.LogicsContracts;
using InternetShopDatabase.Models;
using InternetShopForms.Orders;
using InternetShopForms.Products;
using InternetShopLogics.Logics;
using System.Windows.Forms;

namespace InternetShopForms
{
    public partial class FormOrdersList : Form
    {
        private IOrderLogic _orderLogic;
        public FormOrdersList(IOrderLogic orderLogic)
        {
            InitializeComponent();
            _orderLogic = orderLogic;

            List<(string, string, float)> configureColumns = [
                ("ID", "Id", 1.0f),
                (" ", "CustomerFIO", 1.0f),
                ("", "ProductsString", 1.0f),
                ("Email ", "CustomerEmail", 1.0f),
            ];

            tableComponent.ConfigureColumns(configureColumns);

            tableComponent.dataGridView1.Columns[0].Visible = false;
            KeyPreview = true;
        }

        private void productsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormProductsList));
            if (service is FormProductsList form)
            {
                form.ShowDialog();
            }
        }

        private void FormOrdersList_Load(object sender, EventArgs e)
        {
            LoadData();
        }

        private void LoadData()
        {
            try
            {
                var orders = _orderLogic.ReadList();
                tableComponent.FillData<OrderViewModel>(orders);
            }
            catch (Exception ex)
            {
                MessageBox.Show("    :\n" + ex.Message, "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void FormOrdersList_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control)
            {
                switch (e.KeyCode)
                {
                    case Keys.A:
                        CreateOrder();
                        e.SuppressKeyPress = true;
                        break;
                    case Keys.U:
                        EditOrder();
                        e.SuppressKeyPress = true;
                        break;
                    case Keys.D:
                        DeleteOrder();
                        e.SuppressKeyPress = true;
                        break;
                    case Keys.S:
                        CreateWord();
                        e.SuppressKeyPress = true;
                        break;
                    case Keys.C:
                        CreateExcel();
                        e.SuppressKeyPress = true;
                        break;
                    case Keys.T:
                        CreatePDF();
                        e.SuppressKeyPress = true;
                        break;
                }
            }
        }

        private void CreateOrder()
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormOrderEdit));
            if (!(service is FormOrderEdit form))
            {
                return;
            }

            if (form.ShowDialog() == DialogResult.OK)
            {
                LoadData();
            }
        }

        private void EditOrder()
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormOrderEdit));
            if (!(service is FormOrderEdit form))
            {
                return;
            }

            var selectedOrder = tableComponent.GetSelectedObject<OrderViewModel>();
            form.OrderId = Convert.ToInt32(selectedOrder.Id);
            if (form.ShowDialog() == DialogResult.OK)
            {
                LoadData();
            }
        }

        private void DeleteOrder()
        {
            try
            {
                var selectedOrder = tableComponent.GetSelectedObject<OrderViewModel>();

                if (selectedOrder != null)
                {
                    var result = MessageBox.Show(
                        "   ?",
                        "",
                        MessageBoxButtons.YesNo,
                        MessageBoxIcon.Question
                    );

                    if (result == DialogResult.Yes)
                    {
                        _orderLogic.Delete(new OrderSearchModel { Id = selectedOrder.Id });
                        MessageBox.Show(" ");
                        LoadData();
                    }
                    LoadData();
                }
                LoadData();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void CreateWord()
        {
            // todo
        }

        private void CreateExcel()
        {
            // todo
        }

        private void CreatePDF()
        {
            // todo
        }
    }
}
